@import url("synfig_dialog.css");

#render-progress * {
	min-height: 30px;
}

/*
 * Make progress bar noticeable and align it correctly on the status bar
 */
#status-progress * {
	min-height   : 18px;
	margin-right : 8px;
	margin-bottom: 3px;
}

/*
 * Toolbar buttons are too big for default Adwaita theme
 * Make it small
 */
.synfigstudio-efficient-workspace button {
	padding: 0px;
}

/*
 * The above sets button padding to 0
 * That means buttons are tightly cramped together without spaces
 * Set button's image padding to 5 px to give consistent top-right-bottom-left space
 */
.synfigstudio-efficient-workspace button > image {
	padding: 5px;
}

/*
 * Image padding for buttons are set
 * Now set padding for text inside buttons
 */
#low-resolution label {
	padding-left : 5px;
	padding-right: 5px;
}

/*
 * Make spinbuttons and entries look consistent with small toolbar buttons
 */
.synfigstudio-efficient-workspace entry, spinbutton {
	min-height: 16px;
}

/*
 * We don't want the combobox icons touching the left/right edge.
 * Give some lil' space.
 */
.synfigstudio-efficient-workspace combobox button {
	padding-left : 5px;
	padding-right: 5px;
}

/*
 * Our timetrack ruler is tight
 * Set some padding below to give it more space
 */
#timetrack_panel button {
	padding-bottom: 5px;
}

/*
 * Synfig Studio dialog box
 */
.dialog-main-content, .color-grid {
	margin-left  : 12px;
	margin-right : 12px;
	margin-top   : 12px;
	margin-bottom: 12px;
}

/*
 * Synfig Studio options in dialogs
 * Slightly moved to the right
 */
.dialog-secondary-content {
	margin-left  : 24px;
	margin-right : 0px;
	margin-top   : 6px;
	margin-bottom: 0px;
}

.gap {
	margin-right : 3px;
}

.indentation{
	margin-left : 6px;
}

/*
 * Let WorkArea/Canvas background be the
 * default system background
 */
#canvasview {
	background: @theme_unfocused_bg_color;
}

.color-red {
	color : #FF0000;
}

.color-white {
	color : #FFFFFF;
}

#parameters_panel .odd,
#timetrack_panel .odd,
#layers_panel .odd {
	background-color: rgba(255,255,255,0.05);
}

#parameters_panel .even,
#timetrack_panel .even,
#layers_panel .even {
	background-color: rgba(0,0,0,0.05);
}